package gs

import sbt._

/** Helper for managing all SBT command line properties (`-Dproperty=value`).
  */
object PluginProperties {

  /** Helper to extract the value from `-Dproperty=value`.
    *
    * @param name
    *   The property name.
    * @param conv
    *   The conversion function to the output type.
    * @return
    *   The converted value, or `None` if no value exists.
    */
  def getProperty[A](
    name: String,
    conv: String => A
  ): Option[A] =
    Option(System.getProperty(name)).map(conv)

    /** Use one of the following to calculate an incremented release version:
      *
      *   - `sbt -Drelease=major`
      *   - `sbt -Drelease=minor`
      *   - `sbt -Drelease=patch`
      */
    val ReleaseProperty: String = "release"

    /** If true, or no release type is set, this build is treated as a
      * pre-release, and the suffix `-SNAPSHOT` is appended to the version.
      */
    val SnapshotProperty: String = "snapshot"

    /** The value of `-Drelease=<value>`, parsed and validated. Influences the
      * final calculated [[SemVer]].
      *
      * @return
      *   The version passed as input to SBT.
      */
    def getReleaseType(): Option[ReleaseType] =
      getProperty(
        ReleaseProperty,
        raw =>
          ReleaseType.parse(raw) match {
            case scala.util.Success(value) => value
            case scala.util.Failure(e)     => throw e
          }
      )

    /** The value of `-Dsnapshot=<value>`, parsed and validated. If parsing
      * fails or no value is present, `false` is returned. Influences whether
      * the pre-release suffix (`-SNAPSHOT`) is appended to the version.
      *
      * @return
      *   `true` if `-Dsnapshot=true` is specified at the command line, `false`
      *   otherwise.
      */
    def isSnapshot(): Boolean =
      getProperty(SnapshotProperty, raw => raw.trim().equalsIgnoreCase("true"))
        .getOrElse(false)

}
